pref("extensions.savedpasswordeditor.display_menuitem", true);
pref("extensions.savedpasswordeditor.rename_menuitem_to", "");
pref("extensions.savedpasswordeditor.confirm_ctxmenu_delete", true);
pref("extensions.savedpasswordeditor.openspkey", "");
pref("extensions.savedpasswordeditor.openspkeycode", 0);
pref("extensions.savedpasswordeditor.openspkeymodifiers", "");
pref("extensions.savedpasswordeditor.preselect_current_site", false);
pref("extensions.savedpasswordeditor.always_show_passwords", false);
pref("extensions.savedpasswordeditor.force_prompt_for_masterPassword", true);
pref("extensions.savedpasswordeditor.showpassword", 0);
