/*
    Saved Password Editor, extension for Gecko applications
    Copyright (C) 2014  Daniel Dawson <danielcdawson@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

@namespace "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

#savedpasswordeditor-button {
  list-style-image: url('chrome://savedpasswordeditor/skin/key24.png');
}

#savedpasswordeditor-toolsmenuitem, #savedpasswordeditor-appmenuitem,
toolbar[iconsize="small"] #savedpasswordeditor-button {
  list-style-image: url('chrome://savedpasswordeditor/skin/key16.png');
}

/* Australis */
#savedpasswordeditor-button[cui-areatype="menu-panel"],
toolbarpaletteitem[place="palette"] > #savedpasswordeditor-button {
  list-style-image: url('chrome://savedpasswordeditor/skin/key32.png');
}
