/*
    Saved Password Editor, extension for Gecko applications
    Copyright (C) 2011  Daniel Dawson <danielcdawson@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

@namespace "http://www.w3.org/1999/xhtml";

html {
  direction: ltr;
}

html[dir="RTL"] {
  direction: rtl;
}

body {
  margin-left: 0px;
  margin-right: 0px;
  padding-left: 96px;
  padding-right: 16px;
  background-color: #DDE;
  color: black;
}

html[dir="RTL"] > body {
  padding-left: 16px;
  padding-right: 96px;
}

h1:before {
  margin-left: -80px;
  margin-right: 16px;
  float: left;
  content: url("chrome://savedpasswordeditor/skin/key64.png");
}

html[dir="RTL"] h1:before {
  margin-left: 16px;
  margin-right: -80px;
  float: right;
}

p {
  text-align: justify;
}
